@@ -128,6 +128,20 @@ typedef struct _outputContext
 	int			gzOut;
 } OutputContext;
 
+typedef enum
+{
+	SQL_SCAN = 0,				/* normal */
+	SQL_IN_SINGLE_QUOTE,		/* '...' literal */
+	SQL_IN_DOUBLE_QUOTE			/* "..." identifier */
+} sqlparseState;
+
+typedef struct
+{
+	sqlparseState state;		/* see above */
+	bool		backSlash;		/* next char is backslash quoted? */
+	PQExpBuffer curCmd;			/* incomplete line (NULL if not created) */
+} sqlparseInfo;
+
 typedef enum
 {
 	STAGE_NONE = 0,
@@ -136,6 +150,13 @@ typedef enum
 	STAGE_FINALIZING
 } ArchiverStage;
 
+typedef enum
+{
+	OUTPUT_SQLCMDS = 0,			/* emitting general SQL commands */
+	OUTPUT_COPYDATA,			/* writing COPY data */
+	OUTPUT_OTHERDATA			/* writing data as INSERT commands */
+} ArchiverOutput;
+
 typedef enum
 {
 	REQ_SCHEMA = 1,
@@ -163,6 +184,8 @@ typedef struct _archiveHandle
 								 * Added V1.7 */
 	ArchiveFormat format;		/* Archive format */
 
+	sqlparseInfo sqlparse;		/* state for parsing INSERT data */
+
 	time_t		createDate;		/* Date archive created */
 
 	/*
@@ -208,7 +231,7 @@ typedef struct _archiveHandle
 	PGconn	   *connection;
 	int			connectToDB;	/* Flag to indicate if direct DB connection is
 								 * required */
-	bool		writingCopyData;	/* True when we are sending COPY data */
+	ArchiverOutput outputKind;	/* Flag for what we're currently writing */
 	bool		pgCopyIn;		/* Currently in libpq 'COPY IN' mode. */
 
 	int			loFd;			/* BLOB fd */
